---
title: K-Means - Une Implémentation
author: Boris Stefanovic
date: 2022-05-24
theme: "Frankfurt"
geometry: "margin=40mm"
mainfont: DejaVu Sans
header-includes:

- \usepackage{float}
- \let\origfigure\figure
- \let\endorigfigure\endfigure
- \renewenvironment{figure}[1][2] {\expandafter\origfigure\expandafter[H]} {\endorigfigure}

---

# Structures de Données

## Vecteur

- chaque point est un vecteur
- types entiers et virgule flottante séparés
- "common.h" contient les définitions de `int_t` et `fpt_t`

We can justify the implementation of vectors for multiple types to ease
application to several scenarios without the need for casting,
e.g. scientific measurements (floating point) and image data (integer).

```c
typedef struct vector_int {
	size_t dim;
	int_t* data;
} vector_int_t;

typedef struct vector_fpt {
	size_t dim;
	fpt_t* data;
} vector_fpt_t;
```

## Faciliter l'Association à un Cluster

- structure vecteur vue précédemment générale
- on veut associer chaque point à un cluster
- le cluster auquel chaque point appartient change au cours de l'algorithme
	- stocker les points "dans" des structures "cluster" est peu judicieux
	- on stocke un identifiant de cluster (un pointeur) dans une structure "point de cluster"

## Cluster

- un cluster peut être représenté par
	- un identifiant: un pointeur, forcémment unique
	- son centre: un point virtuel, la valeur derrière le pointeur

```c
typedef vector_int_t* cluster_int_t;
```

## Point de Cluster

```c
typedef struct point_int {
	vector_int_t* vector;
	vector_int_t* cluster;
} point_int_t;
```

On justifie la présence d'une adresse de centre dans la struct par
le fait qu'il y ait une relation qui à chaque centre associe probablement
plusieurs points.

## Ensemble de Points

- parcours répétés de l'ensemble de tous les points
- pas d'ordre particulier (sauf à l'initialisation des centroïdes)
- une liste simplement chaînée fera l'affaire

```c
typedef struct list_points_node_int {
	const point_int_t* point;
	struct list_points_node_int* next;
} list_points_node_int_t;

typedef struct list_points_int {
	list_points_node_int_t* head;
	list_points_node_int_t* tail;
	size_t size;
} list_points_int_t;
```

# Algorithmique

# Implémentation

# Démonstration
