#include "distance.h"
#include <math.h>
#include "common.h"

#define ERROR -1.0


int_t abs_diff_int(const int_t a1, const int_t a2) {
	int_t diff = a2 - a1;
	return diff >= 0 ? diff : -diff;
}

fpt_t abs_diff_fpt(const fpt_t a1, const fpt_t a2) {
	fpt_t diff = a2 - a1;
	return diff >= 0.0 ? diff : -diff;
}


fpt_t distance_euclid_int(const vector_int_t* p1, const vector_int_t* p2) {
	if (p1->dim != p2->dim) return ERROR;
	int_t acc = 0;
	for (size_t i = 0; i < p1->dim; ++i) {
		int_t diff = p2->data[i] - p1->data[i];
		int_t item = diff * diff;
		acc += item;
	}
	return sqrt((fpt_t) acc);
}

fpt_t distance_euclid_fpt(const vector_fpt_t* p1, const vector_fpt_t* p2) {
	if (p1->dim != p2->dim) return ERROR;
	fpt_t acc = 0;
	for (size_t i = 0; i < p1->dim; ++i) {
		fpt_t diff = p2->data[i] - p1->data[i];
		fpt_t item = diff * diff;
		acc += item;
	}
	return sqrt((fpt_t) acc);
}


fpt_t distance_manhattan_int(const vector_int_t* p1, const vector_int_t* p2) {
	if (p1->dim != p2->dim) return ERROR;
	int_t acc = 0;
	for (size_t i = 0; i < p1->dim; ++i) {
		int_t diff = p2->data[i] - p1->data[i];
		int_t item = diff >= 0 ? diff : -diff;
		acc += item;
	}
	return (fpt_t) acc;
}

fpt_t distance_manhattan_fpt(const vector_fpt_t* p1, const vector_fpt_t* p2) {
	if (p1->dim != p2->dim) return ERROR;
	fpt_t acc = 0;
	for (size_t i = 0; i < p1->dim; ++i) {
		fpt_t diff = p2->data[i] - p1->data[i];
		fpt_t item = diff >= 0 ? diff : -diff;
		acc += item;
	}
	return (fpt_t) acc;
}


fpt_t distance_chebyshev_int(const vector_int_t* p1, const vector_int_t* p2) {
	if (p1->dim != p2->dim) return ERROR;
	int_t max = ERROR;
	int_t item;
	for (size_t i = 0; i < p1->dim; ++i) {
		item = abs_diff_int(p1->data[i], p2->data[i]);
		if (item > max) max = item;
	}
	return (fpt_t) max;
}

fpt_t distance_chebyshev_fpt(const vector_fpt_t* p1, const vector_fpt_t* p2) {
	if (p1->dim != p2->dim) return ERROR;
	fpt_t max = ERROR;
	fpt_t item;
	for (size_t i = 0; i < p1->dim; ++i) {
		item = abs_diff_fpt(p1->data[i], p2->data[i]);
		if (item > max) max = item;
	}
	return (fpt_t) max;
}
