#define _GNU_SOURCE

#include "io.h"
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "linkedlist.h"
#include "vector.h"


int_t io_read_int(FILE* file) {
	char* line = NULL;
	size_t len = 0;
	getline(&line, &len, file);
	long res = strtol(line, NULL, 10);
	free(line);
	return res;
}

fpt_t io_read_fpt(FILE* file) {
	char* line = NULL;
	size_t len = 0;
	getline(&line, &len, file);
	double res = strtod(line, NULL);
	free(line);
	return res;
}


vector_int_t* io_line_to_vector_int(char* line, const size_t dim) {
	vector_int_t* vector = vector_create_int(dim);
	char* tgt = line;
	char* token = NULL;
	for (size_t i = 0; i < dim; ++i, tgt = NULL) {
		token = strtok(tgt, ",");
		// strtol returns 0 if number not read, which is the desired behaviour:
		vector->data[i] = token != NULL ? strtol(token, NULL, 10) : 0;
	}
	return vector;
}

vector_fpt_t* io_line_to_vector_fpt(char* line, const size_t dim) {
	vector_fpt_t* vector = vector_create_fpt(dim);
	char* tgt = line;
	char* token = NULL;
	for (size_t i = 0; i < dim; ++i, tgt = NULL) {
		token = strtok(tgt, ",");
		// strtol returns 0 if number not read, which is the desired behaviour:
		vector->data[i] = token != NULL ? strtod(token, NULL) : 0;
	}
	return vector;
}


list_points_int_t* io_get_vector_list_int(FILE* ifile, const size_t dim) {
	list_points_int_t* list = list_points_create_int();
	char* line = NULL;
	size_t len = 0;
	while (getline(&line, &len, ifile) != -1) {
		if (len != 0) {
			vector_int_t* vector = io_line_to_vector_int(line, dim);
			list_points_append_int(list, vector);
			//free(line);
		}
	}
	free(line);
	return list;
}

list_points_fpt_t* io_get_vector_list_fpt(FILE* ifile, const size_t dim) {
	list_points_fpt_t* list = list_points_create_fpt();
	char* line = NULL;
	size_t len = 0;
	while (getline(&line, &len, ifile) != -1) {
		if (len != 0) {
			vector_fpt_t* vector = io_line_to_vector_fpt(line, dim);
			list_points_append_fpt(list, vector);
			//free(line);
		}
	}
	free(line);
	return list;
}


void io_write_clusters_to_file_int(FILE* file, cluster_int_t** clusters, const size_t cluster_count) {
	for (size_t i = 0; i < cluster_count; ++i) {
		fprintf(file, "\n*\n");
		list_points_node_int_t* node = clusters[i]->points->head;
		while (node != NULL) {
			const vector_int_t point = *(node->point);
			fprintf(file, "%ld", point.data[0]);
			for (size_t p = 1; p < point.dim; ++p) {
				fprintf(file, " , %ld", point.data[p]);
			}
			fprintf(file, "\n");
			node = node->next;
		}
	}
}

void io_write_clusters_to_file_fpt(FILE* file, cluster_fpt_t** clusters, const size_t cluster_count) {
	for (size_t i = 0; i < cluster_count; ++i) {
		fprintf(file, "\n*\n");
		list_points_node_fpt_t* node = clusters[i]->points->head;
		while (node != NULL) {
			const vector_fpt_t point = *(node->point);
			fprintf(file, "%lf", point.data[0]);
			for (size_t p = 1; p < point.dim; ++p) {
				fprintf(file, " , %lf", point.data[p]);
			}
			fprintf(file, "\n");
			node = node->next;
		}
	}
}
