#include "vector.h"
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>


vector_int_t* vector_create_int(const size_t dim) {
	vector_int_t* v;
	if ((v = malloc(sizeof(vector_int_t))) == NULL) return NULL;
	v->dim = dim;
	v->data = calloc(dim, sizeof(int_t));
	return v;
}

vector_fpt_t* vector_create_fpt(const size_t dim) {
	vector_fpt_t* v;
	if ((v = malloc(sizeof(vector_fpt_t))) == NULL) return NULL;
	v->dim = dim;
	v->data = calloc(dim, sizeof(fpt_t));
	return v;
}


void vector_destroy_int(vector_int_t* vp) {
	if (NULL == vp) return;
	free(vp->data);
	free(vp);
}

void vector_destroy_fpt(vector_fpt_t* vp) {
	if (NULL == vp) return;
	free(vp->data);
	free(vp);
}


vector_int_t* vector_copy_int(const vector_int_t* v) {
	if (NULL == v) return NULL;
	vector_int_t* c = vector_create_int(v->dim);
	if (NULL == c) return NULL;
	for (size_t i = 0; i < v->dim; ++i) {
		c->data[i] = v->data[i];
	}
	return c;
}

vector_fpt_t* vector_copy_fpt(const vector_fpt_t* v) {
	if (NULL == v) return NULL;
	vector_fpt_t* c = vector_create_fpt(v->dim);
	if (NULL == c) return NULL;
	for (size_t i = 0; i < v->dim; ++i) {
		c->data[i] = v->data[i];
	}
	return c;
}


bool vector_equals_int(const vector_int_t* v1, const vector_int_t* v2) {
	if (v1->dim != v2->dim) return false;
	for (size_t i = 0; i < v1->dim; ++i) {
		if (v1->data[i] != v2->data[i]) {
			return false;
		}
	}
	return true;
}

bool vector_equals_fpt(const vector_fpt_t* v1, const vector_fpt_t* v2) {
	if (v1->dim != v2->dim) return false;
	for (size_t i = 0; i < v1->dim; ++i) {
		if (v1->data[i] != v2->data[i]) {
			return false;
		}
	}
	return true;
}


void vector_print_to_file_int(FILE* file, const vector_int_t* v) {
	fprintf(file, "%lud", v->data[0]);
	for (size_t i = 1; i < v->dim; ++i) fprintf(file, " , %lud", v->data[i]);
	fprintf(file, "\n");
}

void vector_print_to_file_fpt(FILE* file, const vector_fpt_t* v) {
	fprintf(file, "%lf", v->data[0]);
	for (size_t i = 1; i < v->dim; ++i) fprintf(file, " , %lf", v->data[i]);
	fprintf(file, "\n");
}


void vector_add_inplace_int(vector_int_t* v, const vector_int_t a) {
	if (NULL == v) return;
	const size_t dim = v->dim < a.dim ? v->dim : a.dim;
	for (size_t i = 0; i < dim; ++i) v->data[i] += a.data[i];
}

void vector_add_inplace_fpt(vector_fpt_t* v, const vector_fpt_t a) {
	if (NULL == v) return;
	const size_t dim = v->dim < a.dim ? v->dim : a.dim;
	for (size_t i = 0; i < dim; ++i) v->data[i] += a.data[i];
}


void vector_div_inplace_int(vector_int_t* v, const int_t a) {
	if (NULL == v) return;
	for (size_t i = 0; i < v->dim; ++i) v->data[i] /= a;
}

void vector_div_inplace_fpt(vector_fpt_t* v, const fpt_t a) {
	if (NULL == v) return;
	for (size_t i = 0; i < v->dim; ++i) v->data[i] /= a;
}


void vector_print_int(const vector_int_t* v) {
	if (NULL == v) printf("NULL");
	else {
		printf("%ld", v->data[0]);
		for (size_t p = 1; p < v->dim; ++p) {
			printf(" , %ld", v->data[p]);
		}
		printf("\n");
	}
}

void vector_print_fpt(const vector_fpt_t* v) {
	if (NULL == v) printf("NULL");
	else {
		printf("%lf", v->data[0]);
		for (size_t p = 1; p < v->dim; ++p) {
			printf(" , %lf", v->data[p]);
		}
		printf("\n");
	}
}
